/*In this example we have entered into a forward agreement
with a counterparty in which we will buy 1000 shares of 
TechCompany stock at a price of 
$20/share in July 2013. There is the possibility the counterparty will
default and we will be exposed to credit losses. We examine the credit risk over the next three days
by simulating exposure and credit losses.*/

libname RDExamp "C:\users\sasdemo\RD_Examples";
%let test_env = creditrisk;

/*------------------------------------------------*/
/*---- 1. Create the environment -----------------*/
/*------------------------------------------------*/
proc risk;
  env new=RDExamp.&test_env;
  env save;
run;

/*------------------------------------------------*/
/*---- 2. Register portfolio data ----------------*/
/*------------------------------------------------*/

data RDExamp.instdata; 
  format maturity date9.;
  length insttype $12. cpty_name $16. contract_type $16.;
  input insttype $ instid $ cpty_name $ CounterpartyID $ maturity date9.  
        int_rate  contract_type $ contract_price holding;
  datalines;
FwdContract instid1 CounterPartyA cptyid1 01JUL2014 0.02 TechCompany 20 1000
;

data RDExamp.cptydata; 
  length CounterPartyType $16.  cpartyname $16. rating $3.;
  input CounterPartyType $ CounterpartyID $  cpartyname $ rating $;
  datalines;
MidGrade_Cpty cptyid1 CounterPartyA BBB 
;

proc risk; 
  env open=RDExamp.&test_env;
  instdata Instruments file=RDExamp.instdata format=simple;
  cptydata Counterparties file=RDExamp.cptydata; 
  declare 
	instvars = ( 
		cpty_name char 16 var,
		maturity num var,
		int_rate num var,
		contract_type char 16 var,
		contract_price num var
	)
	cptyvars = (
		rating char 3 var,
		cpartyname char 16 var,
		prob_default num computed,
		loss_default num computed
	)		
      outvars = (creditloss num computed comptype = exposure)
      ;
  env save;
run;



/*------------------------------------------------*/
/*---- 3. Register the market data ---------------*/
/*------------------------------------------------*/

data RDExamp.mktdata;
  TEC = 19.75; 
run;

data RDExamp.covdata;
	length _name_ _type_ $8;
	_type_ = "COV";
	input _name_ $ TEC ;  
	datalines;
TEC      .02    
;

proc risk;
	env open=RDExamp.&test_env;
	declare riskfactors =(TEC num var label = "TechCompany Share Price");
	marketdata Market file=RDExamp.mktdata  type=current;
	marketdata Cov    file=RDExamp.covdata  type=covariance;
	env save;
run;


/*------------------------------------------------*/
/*---- 4. Define valuation methods ---------------*/
/*------------------------------------------------*/

proc compile env=RDExamp.&test_env outlib=RDExamp.&test_env;
  method Score_Method kind = scoring;
  	if rating = 'BBB' then do;
	     prob_default = .10;
		 loss_default = .20;
	end;
	else do;
		 prob_default = .15;
		 loss_default = 1;
	end;
  endmethod;

  method Fwd_Contract kind=price;
	 time_yrs = (maturity - _date_)/365.25;
     curr_fwd = TEC * exp(int_rate * time_yrs);
	 _value_ = (curr_fwd - contract_price)*exp(-1 * int_rate * time_yrs);
	 creditloss = max(0,_value_) * loss_default * prob_default ;
  endmethod;
run;


proc risk;
  env open=RDExamp.&test_env;
  counterparty MidGrade_Cpty variables = (rating, prob_default, loss_default)
               method = Score_Method;
  instrument FwdContract variables=(int_rate, maturity, contract_price,
                                    counterpartyid, holding) 
                                    methods=(Price Fwd_Contract);
  env save;
run;


/*------------------------------------------------*/
/*---- 5. Create Project and Specify Analysis ----*/
/*------------------------------------------------*/

proc risk;
  env open=RDExamp.&test_env;
  sources DataSources (Instruments Counterparties);
  read sources=DataSources out=Portfolio;
  crossclass CreditRiskCC (Insttype);
  simulation Covariance method=covariance
          ndraws=50000
          outvars=(pl exposure creditloss)
          horizon = (1 2 3)
          data=Cov;
  project Project
    analysis=(Covariance)
    portfolio=Portfolio
    crossclass=CreditRiskCC
    data=( Market );
    
  env save;
run;


/*------------------------------------------------*/
/*---- 6. Run Analysis ---------------------------*/
/*------------------------------------------------*/

proc risk;
  env open=RDExamp.&test_env;  
    runproject Project
      out= Credit
      options=(outall);  
  env save;
run;
